@@ -10,6 +10,63 @@
 #' @param path Character. url link to zip file OR path to local zip file. if to local path, then option `local` must be set to TRUE.
 #' @param local Boolean. If the paths are searching locally or not. Default is FALSE (that is, urls).
 #' @param quiet Boolean. Whether to see file download progress and files extract. FALSE by default.
+#' @param frequency Boolean. Whether to add frequency/headway calculations to the gtfs object
+#'
+#' @return A GTFS object. That is, a list of dataframes of GTFS data.
+#'
+#' @export
+#' @importFrom dplyr %>% arrange summarise group_by inner_join
+#' @examples \donttest{
+#' library(dplyr)
+#' u1 <- "https://github.com/r-transit/trread/raw/master/inst/extdata/sample-feed-fixed.zip"
+#' sample_gtfs <- read_gtfs(u1)
+#' attach(sample_gtfs)
+#' #list routes by the number of stops they have
+#' routes_df %>% inner_join(trips_df, by="route_id") %>%
+#'   inner_join(stop_times_df) %>% 
+#'     inner_join(stops_df, by="stop_id") %>% 
+#'       group_by(route_long_name) %>%
+#'         summarise(stop_count=n_distinct(stop_id)) %>%
+#'           arrange(desc(stop_count))
+#' }
+read_gtfs <- function(path, local = FALSE, 
+                      quiet = TRUE, 
+                      frequency=FALSE) {
+  # download zip file
+  if(!local) {
+    path <- download_from_url(url = path, quiet = quiet)
+    if(is.null(path)) { return() }
+  }
+  
+  # extract zip file
+  tmpdirpath <- unzip_file(path, quiet=quiet)
+  
+  file_list_df <- zip::zip_list(path)
+  if(!exists("file_list_df")) {
+    stop(sprintf("No files found in zip"))
+  }
+  
+  gtfs_obj <- create_gtfs_object(tmpdirpath, file_list_df$filename, quiet = quiet)
+  
+  if(frequency) {
+    gtfs_obj <- get_route_frequency(gtfs_obj) 
+  }
+  
+  return(gtfs_obj) 
+}
+
+#' This function is deprecated. Please use read_gtfs
+#' 
+#' This function reads GTFS text files from a local or remote zip file. 
+#' It also validates the files against the GTFS specification by file, requirement status, and column name
+#' The data are returned as a list of dataframes and a validation object, 
+#' which contains details on whether all required files were found, 
+#' and which required and optional columns are present. 
+#' 
+#'
+#' @param path Character. url link to zip file OR path to local zip file. if to local path, then option `local` must be set to TRUE.
+#' @param local Boolean. If the paths are searching locally or not. Default is FALSE (that is, urls).
+#' @param quiet Boolean. Whether to see file download progress and files extract. FALSE by default.
 #'
 #' @return Dataframes of GTFS data.
 #'
@@ -17,34 +74,20 @@
 #' @importFrom dplyr %>% arrange summarise group_by inner_join
 #' @examples \donttest{
 #' library(dplyr)
-#' u1 <- "https://developers.google.com/transit/gtfs/examples/sample-feed.zip"
+#' u1 <- "https://github.com/r-transit/trread/raw/master/inst/extdata/sample-feed-fixed.zip"
 #' sample_gtfs <- import_gtfs(u1)
 #' attach(sample_gtfs)
 #' #list routes by the number of stops they have
 #' routes_df %>% inner_join(trips_df, by="route_id") %>%
-#'   inner_join(stop_times_df) %>% 
-#'     inner_join(stops_df, by="stop_id") %>% 
+#'   inner_join(stop_times_df) %>%
+#'     inner_join(stops_df, by="stop_id") %>%
 #'       group_by(route_long_name) %>%
 #'         summarise(stop_count=n_distinct(stop_id)) %>%
 #'           arrange(desc(stop_count))
 #' }
-
 import_gtfs <- function(path, local = FALSE, quiet = FALSE) {
-  if(local) {
-    path <- normalizePath(path) 
-    data_list <- path %>%
-      unzip_file(quiet=quiet) %>% 
-         list_files(quiet=quiet) %>%
-            read_and_validate()
-  } else {
-    data_list <- path %>%
-      download_from_url(.) %>%
-        unzip_file(quiet = quiet) %>%
-          list_files(quiet = quiet) %>%
-            read_and_validate()
-  }
-
-  return(data_list) 
+  .Deprecated("read_gtfs") #include a package argument, too
+  read_gtfs(path, local = FALSE, quiet = FALSE)
 }
 
 #' Download a zipped GTFS feed file from a url
@@ -72,13 +115,9 @@ download_from_url <- function(url, path=tempfile(fileext = ".zip"), quiet=FALSE)
   }
   
   # check if url links to a zip file
-  valid <- valid_url(url)
-  if(!valid) {
-    if(!quiet) {
-      stop1 <- sprintf("Link '%s' is invalid; failed to connect. NULL was returned.", url)
-      stop(stop1)
-    }
-    return(NULL)
+  if(!valid_url(url)) {
+    stop1 <- sprintf("Link '%s' is invalid; failed to connect.", url)
+    stop(stop1)
   }
   
   r <- httr::GET(url)
@@ -101,13 +140,17 @@ download_from_url <- function(url, path=tempfile(fileext = ".zip"), quiet=FALSE)
   check <- try(normalizePath(path), silent = TRUE)
   if(assertthat::is.error(check)) {
     warn <- 'Invalid file path. NULL is returned.'
-    if(!quiet) warning(warn)
+    warning(warn)
     return(NULL)
   }
   return(path)
 }
 
-#' Checks UTF-8-BOM encoding. Special thanks to @patperu for finding the issue and to @hrbrmstr for the code to help deal with the issue.
+#' Checks UTF-8-BOM encoding.
+#' 
+#' Special thanks to @patperu for finding the issue and 
+#' to @hrbrmstr for the code to help deal with the issue.
+#' 
 #' @param path the path the the text file
 #' @param encoding can be one of \code{UTF-8}, \code{UTF-16} or \code{UTF-16BE}.
 #'        Although a BOM could be used with UTF-32 and other encodings, such
@@ -137,18 +180,19 @@ has_bom <- function(path, encoding="UTF-8") {
 #' Unzip a file and delete zip
 #'
 #' @param zipfile path to zipped file
-#' @param ex_dir path to unzip file to-default tempdir()
+#' @param tmpdirpath path to unzip file to-default tempdir()
 #' @param quiet Boolean. Whether to output files found in folder.
-#'
+#' @importFrom tools file_ext
+#' 
 #' @return file path to directory with gtfs .txt files
 #' @keywords internal
 #' 
 
 unzip_file <- function(zipfile, 
-                       ex_dir=tempdir(), 
+                       tmpdirpath=tempdir(), 
                        quiet = FALSE) {
   f <- zipfile
-
+  
   # check path
   if(try(path.expand(f), silent = TRUE) %>% assertthat::is.error()) {
     warn <- 'Invalid file path. NULL is returned.'
@@ -158,80 +202,26 @@ unzip_file <- function(zipfile,
 
   f <- normalizePath(f)
 
+  if(tools::file_ext(f) != "zip") {
+    if(!quiet) message('No zip file found, reading files from path.')
+    return(f)
+  }
+  
   # create extraction folder
-
-  utils::unzip(f, exdir=ex_dir)
+  utils::unzip(f, exdir=tmpdirpath)
 
 
-  if(length(list.files(ex_dir)) == 0) {
+  if(length(list.files(tmpdirpath)) == 0) {
     warn <- "No files found after decompressing. NULL is returned."
     return(NULL)
   }
 
   if(!quiet) {
-    message(sprintf("Unzipped the following files to directory '%s'...", ex_dir))
-    list.files(ex_dir) %>% print
+    message(sprintf("Unzipped the following files to directory '%s'...", tmpdirpath))
+    list.files(tmpdirpath) %>% print
   }
 
-  return(ex_dir)
-
-}
-
-
-#' Read files with a "txt" suffix in a folder into objects in memory and delete files
-#'
-#' @param ex_dir Character. Path to folder into which files were extracted.
-#' @param quiet Boolean. Whether to output messages and files found in folder.
-#' @keywords internal
-list_files <- function(ex_dir, quiet = FALSE) {
-
-  # check path
-  check <- try(normalizePath(ex_dir), silent=TRUE)
-  if(assertthat::is.error(check)) {
-    warn <- 'Invalid file path. NULL is returned.'
-    if(!quiet) warning(warn)
-    return(NULL)
-  }
-
-  file_list <- list.files(ex_dir, full.names = TRUE)
-  return(file_list)
-}
-
-read_and_validate <- function(all_files, quiet = FALSE) {
-  file_list <- sapply(all_files,get_file_shortname)
-  file_validation_meta <- validate_files(file_list)
-  valid_files_meta <- file_validation_meta %>% 
-    dplyr::filter(spec != 'ext' & provided_status=="yes")
-  valid_filenames <- names(file_list[file_list %in% valid_files_meta$file])
-  exec_env <- environment()
-  
-  lapply(valid_filenames, 
-         function(x) read_gtfs_file(x, 
-                                    assign_envir = exec_env, 
-                                    quiet = quiet))
-
-  ls_envir <- ls(envir = exec_env)
-
-  df_list <- ls_envir[grepl(pattern = '_df', x = ls_envir)]
-
-  gtfs_list <- mget(df_list, envir = exec_env)
-
-  if(!quiet) message('...done.\n\n')
-
-  # check if valid 'gtfs'
-  check <- validate_gtfs_structure(valid_files_meta, gtfs_list, return_gtfs_obj = FALSE, quiet = TRUE)
-  valid <- all(check$all_req_files, check$all_req_fields_in_req_files)
-
-  if(!quiet) message("Testing data structure...")
-  if(valid) {
-    class(gtfs_list) <- 'gtfs'
-    if(!quiet) message("...passed. Valid GTFS object.\n")
-  } else {
-    if(!quiet) message("...failed. Invalid data structure.\n")
-  }
-  gtfs_list$validation <- check 
-  return(gtfs_list)
-  
+  return(tmpdirpath)
 }
 
 #' Function to read all files into dataframes
@@ -242,23 +232,53 @@ read_and_validate <- function(all_files, quiet = FALSE) {
 #' @noRd
 #' @keywords internal
 
-read_gtfs_file <- function(file_path, assign_envir, quiet = FALSE) {
+create_gtfs_object <- function(tmpdirpath, file_paths, quiet = FALSE) {
+  prefixes <- vapply(file_paths,get_file_shortname,FUN.VALUE = "")
+  df_names <- paste(prefixes,"_df",sep="")
+  if(!quiet) message('Reading files in feed...\n')
+  gtfs_obj <- lapply(file_paths, 
+                   function(x) read_gtfs_file(x, 
+                                              tmpdirpath, 
+                                              quiet = quiet))
+  names(gtfs_obj) <- unname(df_names)
+  gtfs_obj[sapply(gtfs_obj, is.null)] <- NULL
+  class(gtfs_obj) <- "gtfs"
+  if(!quiet) message('Reading files in feed... done.\n')
+  
+    
+  gtfs_obj <- validate_gtfs(gtfs_obj, quiet = quiet)
+  
+  stopifnot(is_gtfs_obj(gtfs_obj))
+  
+  if(!quiet) message("Reading gtfs feed completed.\n\n")
+  
+  return(gtfs_obj)
+}
+
+
+#' Function to read all files into dataframes
+#'
+#' @param file_path Character file path
+#' @param tmpdirpath path for the tmpdir files
+#' @param quiet Boolean. Whether to output messages and files found in folder.
+#' @noRd
+#' @keywords internal
+
+read_gtfs_file <- function(file_path, tmpdirpath, quiet = FALSE) {
   prefix <- get_file_shortname(file_path)
-  df_name <- paste0(prefix, '_df')
 
-  if(!quiet) message(paste0('Reading ', df_name))
+  if(!quiet) message(paste0('Reading ', prefix))
 
-  new_df <- parse_gtfs(prefix, file_path, quiet = quiet) 
-  # will have warning even though we fix problem
-
-  assign(df_name, new_df, envir = assign_envir)
+  full_file_path <- paste0(tmpdirpath,"/",file_path)
+  new_df <- parse_gtfs_file(prefix, full_file_path, quiet = quiet)
 
+  return(new_df)
 }
 
 #' Function to get the gtfs table name from the file string
 #'
 #' @param file_path Character file path
-#' @return gtfs_table_name a character vector of file names and their full paths
+#' @return df_name a character vector of the df_name for the file
 #' @noRd
 #' @keywords internal
 #' 
@@ -272,7 +292,7 @@ get_file_shortname <- function(file_path) {
   return(prefix)
 }
 
-#' Function to better read in GTFS txt files
+#' Parses one gtfs file
 #'
 #' @param prefix Character. gtfs file prefix (e.g. 'agency', 'stop_times', etc.)
 #' @param file_path Character. file path
@@ -280,8 +300,9 @@ get_file_shortname <- function(file_path) {
 #' @return Dataframe of parsed GTFS file.
 #' @noRd
 #' @keywords internal
+#' @importFrom data.table fread
 
-parse_gtfs <- function(prefix, file_path, quiet = FALSE) {
+parse_gtfs_file <- function(prefix, file_path, quiet = FALSE) {
 
   # only parse if file has any data, NULL o/w
   stopifnot(!is.na(file.size(file_path)))
@@ -291,47 +312,61 @@ parse_gtfs <- function(prefix, file_path, quiet = FALSE) {
     meta <- get_gtfs_meta()[[prefix]]
 
     # check if a file is empty. If so, return NULL.
-    if(length(scan(file_path, what = "", quiet = TRUE, sep = '\n')) < 1) {
-      s <- sprintf("File '%s' is empty. Returning NULL.\n", basename(file_path))
-      message(s)
-      return()
+    L <- suppressWarnings(length(scan(file_path, what = "", quiet = TRUE, sep = '\n')))
+    if(L < 1) {
+      s <- sprintf("   File '%s' is empty.", basename(file_path))
+      if(!quiet) message(s)
+      return(NULL)
     }
 
     # if no meta data is found for a file type but file is not empty, read as is.
     if(is.null(meta)) {
-      s <- sprintf("File %s not recognized. No meta data exists. Reading file as csv.\n", basename(file_path))
-      message(s)
-      csv <- quote(readr::read_csv(file = file_path))
-      df <- suppressMessages(trigger_suppressWarnings(eval(csv), quiet))
+      s <- sprintf("   File %s not recognized, trying to read file as csv.", basename(file_path))
+      if(!quiet) message(s)
+
+      tryCatch({
+        df <- suppressMessages(data.table::fread(file = file_path, sep=","))
+      }, error = function(error_condition) {
+        s <- sprintf("   File could not be read as csv.", basename(file_path))
+        if(!quiet) message(s)
+        return(NULL)
+      })
       return(df)
     }
 
     ## read.csv supports UTF-8-BOM. use this to get field names.
-    small_df <- suppressWarnings(utils::read.csv(file_path, nrows = 10, stringsAsFactors = FALSE)) # get a small df to find how many cols are needed
+    small_df <- suppressWarnings(utils::read.csv(file_path, nrows = 5, stringsAsFactors = FALSE)) # get a small df to find how many cols are needed
 
     ## get correct coltype, if possible
-    coltypes <- rep('c', dim(small_df)[2]) # create 'c' as coltype defaults
-    names(coltypes) <- names(small_df) %>% tolower()
-    indx <- match(names(coltypes), meta$field)  # indx from valid cols in meta$field. NAs will return for invalid cols
-
-    colnms <- meta$field[indx] # get expected/required names for columns. these are imposed.
+    coltypes_character <- rep('c', dim(small_df)[2]) # create 'c' as coltype defaults
+    names(coltypes_character) <- names(small_df) %>% tolower()
+    indx <- match(names(coltypes_character), meta$field)  # indx from valid cols in meta$field. NAs will return for invalid cols
 
     ## !is.na(indx) = valid col in 'coltype' found in meta$field
     ## indx[!is.na(indx)] = location in 'meta$coltype' where corresponding type is found
-    coltypes[!is.na(indx)] <- meta$coltype[indx[!is.na(indx)]] # valid cols found in small_df
-
-    ## get colclasses for use in read.csv (useful when UTF-8-BOM encoding is found)
-    colclasses <- sapply(coltypes, switch, c = "character", i = "integer", d = "double")
-
-    ## collapse coltypes for use in read_csv
-    coltypes <- coltypes %>% paste(collapse = "")
-
-    ## switch function for when BOMs exist
-    converttype <- function(x, y) {
-      switch(x, character = as.character(y), integer = as.integer(y), double = as.double(y))
-    }
+    coltypes_character[!is.na(indx)] <- meta$coltype[indx[!is.na(indx)]] # valid cols found in small_df
 
+    # use col_*() notation for column types
+    coltypes <-
+      sapply(
+        coltypes_character,
+        switch,
+        "c" = readr::col_character(),
+        "i" = readr::col_integer(),
+        "d" = readr::col_double(),
+        "D" = readr::col_date(format = "%Y%m%d")
+      )
+    
     if (has_bom(file_path)) { # check for BOM. if yes, use read.csv()
+      ## switch function
+      converttype <- function(x, y) {
+        switch(x, character = as.character(y), integer = as.integer(y), double = as.double(y), Date = lubridate::ymd(y))
+      }
+      colnms <- meta$field[indx] # get expected/required names for columns. these are imposed.
+      
+      ## get colclasses
+      colclasses <- sapply(coltypes_character, switch, c = "character", i = "integer", d = "double", "D" = "Date")
+      
       csv <- quote(utils::read.csv(file_path, col.names = colnms, stringsAsFactors= FALSE))
       df <- try(suppressWarnings(eval(csv)) %>%
           mapply(converttype, x = colclasses, y = ., SIMPLIFY = FALSE) %>% # ensure proper column types
@@ -341,14 +376,19 @@ parse_gtfs <- function(prefix, file_path, quiet = FALSE) {
         probs <- "Error during import. Likely encoding error. Note that utils::read.csv() was used, not readr::read_csv()."
         attributes(df) <- append(attributes(df), list(problems = probs))
       }
-
     } else {
-      csv <- quote(readr::read_csv(file = file_path, col_types = coltypes, col_names = colnms, skip = 1L))
-      prob <- quote(readr::problems(readr::read_csv(file = file_path, col_types = coltypes, col_names = colnms, skip = 1L)))
-      df <- trigger_suppressWarnings(eval(csv), quiet)
-      probs <- trigger_suppressWarnings(eval(prob), quiet)
-
-      if(dim(probs)[1] > 0) attributes(df) <- append(attributes(df), list(problems = probs))
+      df <- suppressWarnings(
+        readr::read_csv(file = file_path, 
+          col_types = coltypes
+        )
+      )
+      probs <- readr::problems(df)
+      
+      if(dim(probs)[1] > 0) {
+        attributes(df) <- append(attributes(df), list(problems = probs))
+        warning(paste0("Parsing failures while reading ", prefix))
+        print(probs)
+      }
     }
 
     return(df)
